@@ -206,7 +206,7 @@ _wrap_g_type_from_name(PyGTypeWrapper *_, PyObject *args)
     if (!PyArg_ParseTuple(args, "s:GType.from_name", &type_name))
 	return NULL;
 
-    type = g_type_from_name(type_name);
+    type = _pyg_type_from_name(type_name);
     if (type == 0) {
 	PyErr_SetString(PyExc_RuntimeError, "unknown type name");
 	return NULL;
@@ -373,14 +373,17 @@ pyg_type_from_object(PyObject *obj)
 
     /* handle strings */
     if (PyString_Check(obj)) {
-	type = g_type_from_name(PyString_AsString(obj));
-	if (type == 0)
-	    PyErr_SetString(PyExc_TypeError, "could not find named typecode");
-	return type;
+	gchar *name = PyString_AsString(obj);
+	
+	type = _pyg_type_from_name(name);
+	if (type != 0) {
+	    return type;
+	}
     }
 
     /* finally, look for a __gtype__ attribute on the object */
     gtype = PyObject_GetAttrString(obj, "__gtype__");
+
     if (gtype) {
 	if (gtype->ob_type == &PyGTypeWrapper_Type) {
 	    type = ((PyGTypeWrapper *)gtype)->type;
@@ -856,7 +859,7 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
                  G_VALUE_HOLDS(value, G_TYPE_GSTRING)) {
             GString *string;
             char *buffer;
-            int len;
+            Py_ssize_t len;
             if (PyString_AsStringAndSize(obj, &buffer, &len))
                 return -1;
             string = g_string_new_len(buffer, len);
@@ -1328,8 +1331,9 @@ gclosure_from_pyfunc(PyGObject *object, PyObject *func)
 
     for (l = object->closures; l; l = l->next) {
 	PyGClosure *pyclosure = l->data;
-	if (PyFunction_GetClosure(pyclosure->callback) ==
-		PyFunction_GetClosure(func)) {
+	int res;
+	PyObject_Cmp(pyclosure->callback, func, &res);
+	if (!res) {
 	    return (GClosure*)pyclosure;
 	}
     }
@@ -1637,3 +1641,69 @@ pyg_param_gvalue_as_pyobject(const GValue* gvalue,
 	return pyg_value_as_pyobject(gvalue, copy_boxed);
     }
 }
+
+/**
+ * pyg_type_registration_callback
+ * @gtypename: type name
+ * @callback: function to run
+ *
+ */
+typedef struct {
+    PyGTypeRegistrationFunction callback;
+    gpointer data;
+} CustomTypeData;
+
+void 
+pyg_type_register_custom_callback(const gchar *typename,
+				  PyGTypeRegistrationFunction callback,
+				  gpointer user_data)
+{
+    CustomTypeData *data;
+
+    if (!custom_type_registration)
+	custom_type_registration = g_hash_table_new_full (g_str_hash, g_str_equal,
+							  g_free, g_free);
+
+    data = g_new (CustomTypeData, 1);
+    data->callback = callback;
+    data->data = user_data;
+
+    g_hash_table_insert(custom_type_registration,
+			g_strdup(typename),
+			data);
+}
+
+PyTypeObject *
+pyg_type_get_custom(const gchar *name)
+{
+    CustomTypeData *data;
+    PyTypeObject *retval;
+    
+    if (!custom_type_registration)
+	return NULL;
+
+    data = g_hash_table_lookup(custom_type_registration, name);
+    if (!data)
+	return NULL;
+
+    retval = data->callback(name, data->data);
+
+    g_hash_table_remove(custom_type_registration, name);
+
+    return retval;
+}
+
+GType
+_pyg_type_from_name(const gchar *name)
+{
+    GType type;
+
+    type = g_type_from_name(name);
+    if (type == G_TYPE_INVALID) {
+	pyg_type_get_custom(name);
+	type = g_type_from_name(name);
+    }
+    
+    return type;
+}
+
