@@ -9,7 +9,7 @@ class TestSubType(unittest.TestCase):
         t = type('testtype', (GObject, GInterface), {})
         self.failUnless(issubclass(t, GObject))
         self.failUnless(issubclass(t, GInterface))
-        
+
     def testGObject(self):
         label = gobject.GObject()
         self.assertEqual(label.__grefcount__, 1)
@@ -29,9 +29,24 @@ class TestSubType(unittest.TestCase):
         self.assertEqual(obj.__grefcount__, 1)
         refcount = testhelper.test_g_object_new()
         self.assertEqual(refcount, 2)
-        
+
     def testRegisterArgNotType(self):
         self.assertRaises(TypeError, gobject.type_register, 1)
 
     def testGObjectNewError(self):
         self.assertRaises(TypeError, gobject.new, gobject.GObject, text='foo')
+
+    def testSubSubType(self):
+        Object1 = type('Object1', (gobject.GObject,),
+                       {'__gtype_name__': 'Object1'})
+        Object2 = type('Object2', (Object1,),
+                       {'__gtype_name__': 'Object2'})
+
+        obj = Object2()
+        self.failUnless(isinstance(obj, Object2))
+        self.assertEqual(obj.__gtype__.name, 'Object2')
+
+        obj = gobject.new(Object2)
+        self.failUnless(isinstance(obj, Object2))
+        self.assertEqual(obj.__gtype__.name, 'Object2')
+
